! module for handling different sets of coordinates (cartesian, orbital...)
module coordinates

  implicit none

  double precision,parameter:: PI = 3.141592653589793238462643d0
  double precision,parameter:: DPI = 2.d0*PI

contains
!***************************************************************************************

  ! Performs the change of coordinates from Cartesian elements to Keplerian coordinates
  !
  subroutine CarToKep(mu, xcar, xkep)

     implicit none

     double precision,intent(in):: mu         ! G*(m0+mi)
     double precision, intent(in):: xcar(6)
     double precision, intent(out):: xkep(6)

     double precision:: x(3), v(3), angm(3), lenz(3), norb(3)
     double precision:: v2, r, em, ener, nm
     double precision:: e3(3), ascnod(3), aux(3)
     double precision:: axi, ecc, inc, omnod, omega, Ecan, mean
     double precision:: cosw, sinw, cosE, sinE

     !------------------------------------------------
     x = xcar(1:3)                                                  ! cartesian position
     v = xcar(4:6)                                                  ! cartesian velocity
     v2 = prscal(v,v)                                               
     r = sqrt(prscal(x,x))                                          
     !------------------------------------------------
     call prvec(x,v,angm)                                           ! angular momentum vector 
     em = sqrt(prscal(angm,angm))
     call prvec(v,angm,lenz)                                         
     lenz = lenz/mu-x/r                                             ! Lenz vector
     norb = angm/em                                                 ! orbit normal unit vector
     ener = v2/2-mu/r                                               ! energy
     !------------------------------------------------
     axi = -mu/(2.d0*ener)                                          ! semi-amjor axis
     nm=sqrt(mu/axi**3)                                             ! mean motion
     ecc = sqrt(prscal(lenz,lenz))                                  ! eccentricity
     inc = acos(norb(3))                                            ! inclination
     !------------------------------------------------
     e3 = (/0,0,1/)
     call prvec(e3,angm,ascnod)
     ascnod = ascnod/sqrt(prscal(ascnod,ascnod))                    ! ascending node unit vector
     omnod = datan2(ascnod(2),ascnod(1))                            ! longitude of the node
     !------------------------------------------------
     call prvec(ascnod,lenz/ecc,aux)
     cosw = prscal(lenz,ascnod)/ecc
     sinw = sqrt(prscal(aux,aux))
     ! WARNING: this works for inclination in (0,90) degrees
     if (aux(3).lt.0) then
        sinw=-sinw
     endif
     omega = datan2(sinw,cosw)                                      ! argument of the pericenter
     !------------------------------------------------
     cosE = 1.d0/ecc*(1.d0-r/axi)
     sinE = prscal(x,v)/(axi**2*ecc*nm)
     Ecan = datan2(sinE,cosE)                                       ! eccentric anomaly
     mean = Ecan-ecc*sinE                                           ! mean anomaly
     !------------------------------------------------
     xkep(1) = axi                            ! semi-amjor axis
     xkep(2) = ecc                            ! eccentricity
     xkep(3) = inc                            ! inclination
     xkep(4) = modulo(omega,DPI)              ! pericenter argument
     xkep(5) = modulo(omnod,DPI)              ! longitude of the node
     xkep(6) = modulo(mean,DPI)               ! mean anomaly 

  end subroutine CarToKep

!***************************************************************************************

  ! scalar product
  function prscal(a,b)

     implicit none

     double precision:: prscal
     double precision, intent(in):: a(3), b(3)

     prscal=a(1)*b(1)+a(2)*b(2)+a(3)*b(3)

  end function prscal

!***************************************************************************************

  ! vector product
  subroutine prvec(a,b,c)

     implicit none

     double precision, intent(in):: a(3), b(3)
     double precision, intent(out):: c(3)

     c(1) = a(2)*b(3)-a(3)*b(2)
     c(2) = a(3)*b(1)-a(1)*b(3)
     c(3) = a(1)*b(2)-a(2)*b(1)

  end subroutine prvec

!***************************************************************************************

end module coordinates
