! Simple program for the propagation of the (N+1)-body problem
! course of Dinamica del Sistema Solare (UniPi)
program nbodyDSS

  use hamiltonian
  use integr

  implicit none

  character(256):: hamParamFile ! namelist containing the parameters of the Hamiltonian function
  character(256):: intParamFile ! namelist containing the parameters of the numerical integration
  character(256):: outFile      ! output file

  type(hamparam):: Hp  ! parameters of the Hamiltonian
  type(intparam):: Ip  ! parameters of the numerical integration

  double precision,allocatable:: X(:) ! state vector
  double precision:: t ! time
  integer:: Nstep, Nreg

!---------------------------------------------------------------------------------------------
! Reading and computing the parameters

  if(COMMAND_ARGUMENT_COUNT() >= 3) then
     call GET_COMMAND_ARGUMENT(1,hamParamFile)
     call GET_COMMAND_ARGUMENT(2,intParamFile)
     call GET_COMMAND_ARGUMENT(3,outFile)
  else
     write(*,*) "Use: <program name> <parameterFile.nml> <initialConditions.nml> <outFile.dat>"
     stop
  endIf

  ! computing the parameters of the Hamiltonian
  call readHamParam(hamParamFile,trim(outFile)//"_ham.nml",Hp)

  allocate(X(6*Hp%Nbod)) ! allocation of the state vector according to the number of bodies

  ! computing the parameters of the numerical integration
  call initInteg(intParamFile,trim(outFile)//"_int.nml", Hp,Ip, X)

!---------------------------------------------------------------------------------------------
! Initializations

  t = Ip%t0
  Nstep = 0 ! number of timesteps realised

  ! Opening and writing the header of the output file
  open(newunit=Nreg,file=trim(outFile),status='REPLACE',action='WRITE') ! opening the output file
  write(Nreg,'(A)') "# a is the semi-major axis"
  write(Nreg,'(A)') "# e is the eccentricity"
  write(Nreg,'(A)') "# I is the inclination"
  write(Nreg,'(A)') "# w is the argument of pericenter"
  write(Nreg,'(A)') "# O is the longitude of ascending node"
  write(Nreg,'(A)') "# l is the mean anomaly"
  write(Nreg,'(A)') "# The angles are expressed in radians."
  write(Nreg,'(A)') "# "

  write(Nreg,'(A,6(I0,A))') "# t ; a(1:",Hp%Nbod,"), e(1:",Hp%Nbod,"), I(1:",Hp%Nbod,&
                                  "), w(1:",Hp%Nbod,"), O(1:",Hp%Nbod,"), l(1:",Hp%Nbod,") ; hamiltonian value"

!---------------------------------------------------------------------------------------------
! Numerical integration

  call printFunc(X,t) ! saving the initial state
  call rkg(X, t, force, Ip%Dt, Ip%tSpan, printFunc) ! numerical integration

!---------------------------------------------------------------------------------------------

  ! Freeing the allocated memory
  call freeHamParam(Hp)
  deallocate(X)

  close(Nreg) ! closing the output file

!---------------------------------------------------------------------------------------------
contains
!---------------------------------------------------------------------------------------------

  ! Force function for the numerical integration
  ! 
  function force(XX,tt)
     double precision,intent(in):: XX(:), tt
     double precision:: force(size(XX))

     double precision:: hamEq(6,Hp%Nbod)
     integer:: j

     ! NOTE: N-body problem does not depend explicitely from the time tt

     ! Conservative part of the force.
     hamEq(:,:) = DHam(Hp,XX(1:6*Hp%Nbod))

     do j = 1,Hp%Nbod
        force(6*j-5:6*j) = hamEq(:,j)
     enddo

     ! other effects (e.g. dissipation) can be added here
     !....

  end function force

!---------------------------------------------------------------------------------------------

  ! Subroutine used after each step for saving the results.
  ! 
  subroutine printFunc(XX,tt)
     double precision,intent(in):: XX(:), tt

     double precision:: XC(6),XK(6),v0(3)
     double precision:: H, a(Hp%Nbod), e(Hp%Nbod), I(Hp%Nbod), w(Hp%Nbod), O(Hp%Nbod), l(Hp%Nbod)
     integer:: j

     if( mod(Nstep,Ip%Nsave) == 0 ) then

        H = Ham(Hp,XX)

        ! integration is in canonical heliocentric/planetocentric coordinates
        ! in output we consider classic heliocentric/planetocentric coordinates
        v0=0.d0
        do j=1,Hp%Nbod
           v0=v0-XX(6*j-2:6*j)
        endDo
        v0=v0/Hp%m0

        ! output in keplerian elements
        do j=1,Hp%Nbod
           XC(1:3) = XX(6*j-5:6*j-3)                ! 0-centric position
           XC(4:6) = XX(6*j-2:6*j)/Hp%mBod(j) - v0  ! 0-centric velocity
           call CarToKep(Hp%GmJmibi(j)/Hp%bi(j), XC, XK)
           a(j) = XK(1)
           e(j) = XK(2)
           I(j) = XK(3)
           w(j) = XK(4)
           O(j) = XK(5)
           l(j) = XK(6)
        enddo

        write(Nreg,'(ES22.15,*(1X,ES22.15))') tt, a(:), e(:), I(:), w(:), O(:), l(:), H

     endIf
     Nstep = Nstep + 1

  end subroutine printFunc

!---------------------------------------------------------------------------------------------

end program nbodyDSS
