! module for handling input files and define hamiltonian and integration parameters
module parameters

  use coordinates
  implicit none

  private:: NbodMAX

  ! Maximum number of bodies allowed
  integer,parameter:: NbodMAX = 20

  !-------------------------------------------------------------------------
  ! Parameters required for computing the Hamiltonian function
  ! 
  type hamparam

     integer:: Nbod ! number of bodies in the system

     double precision:: Mtot                   ! total mass
     double precision:: m0                     ! mass of central body
     double precision,allocatable:: mBod(:)    ! mass of the celestial bodies

     double precision,allocatable:: bi(:)      ! betai           (reduced mass)
     double precision,allocatable:: GmJmibi(:) ! G*(mJ+mi)*betai (2-body part)

     double precision,allocatable:: Gmimj(:,:) ! G*mi*mj (mutual perturbations)

     double precision,allocatable:: GmJmiRJ2J2(:) ! G*mJ*mi*RJ**2*J2 (J2 contribution)

  end type hamparam
!-------------------------------------------------------------------------
  ! Parameters of the numerical integration
  ! 
  type intparam

     double precision:: t0    ! initial time
     double precision:: tSpan ! duration
     double precision:: Dt    ! time step
     integer:: Nsave          ! number of timesteps between each save

  end type intparam

!-------------------------------------------------------------------------
contains
!***************************************************************************************

  ! Read the hamiltonian parameters from an input file and rewrite them in an output file.
  ! All the parameters should have the same system of units (e.g. au, years, Sun-mass).
  ! 
  subroutine readHamParam(ficIn,ficOut,Hp)
     character(len=*),intent(in):: ficIn, ficOut
     type(hamparam),intent(out):: Hp

     integer:: Nbod                   ! number of celestial bodies
     double precision:: Gconst        ! gravitational constant
     double precision:: m0            ! mass of central body
     double precision:: mBod(NbodMAX) ! masses of the bodies
     double precision:: J2            ! J2 of central body
     double precision:: R0            ! equatorial radius of central body

     integer:: Nreg, i, j

     namelist /physicalparams/ Nbod, Gconst, m0, mBod, J2, R0

     !-----------------------------------------------------------------------
     ! Reading the parameters
     open(newunit=Nreg,file=trim(ficIn),status="OLD",action="READ")
     read(Nreg,physicalparams)
     close(Nreg)

     ! Writing the parameters
     open(newunit=Nreg,file=trim(ficOut),status="REPLACE",action="WRITE")

     write(Nreg,'(A)')              "&physicalparams"
     write(Nreg,'(A,I0,A)')         "   Nbod =  ", Nbod,","
     write(Nreg,'(A,ES22.15,A)')    " Gconst = " , Gconst,","
     write(Nreg,'(A,ES22.15,A)')    "     m0 = " , m0,","
     write(Nreg,'(A,*(ES22.15,A))') "   mBod = " , (mBod(i),",",i=1,Nbod)
     write(Nreg,'(A,ES22.15,A)')    "     J2 = " , J2,","
     write(Nreg,'(A,ES22.15,A)')    "     R0 = " , R0,","
     write(Nreg,'(A)') "/"

     close(Nreg)
     !-----------------------------------------------------------------------

     Hp%Nbod = Nbod

     allocate( Hp%bi(Nbod), Hp%GmJmibi(Nbod), Hp%Gmimj(Nbod,Nbod) ) ! mass combinations
     allocate( Hp%mBod(Nbod), Hp%GmJmiRJ2J2(Nbod) )

     do i = 1,Nbod

        Hp%mBod(i) = mBod(i)
        Hp%bi(i) = m0*mBod(i)/(m0+mBod(i))
        Hp%GmJmibi(i) = Gconst*(m0+mBod(i))*Hp%bi(i)
        do j = 1,Nbod
           Hp%Gmimj(i,j) = Gconst*mBod(i)*mBod(j)
        endDo
        Hp%GmJmiRJ2J2(i) = Gconst*m0*mBod(i)*R0**2*J2

     endDo

     Hp%m0 = m0
     Hp%Mtot = m0
     do j = 1,Nbod
        Hp%Mtot = Hp%Mtot + mBod(j)
     endDo

  end subroutine readHamParam

!***************************************************************************************

  ! Read the parameters of the numerical integration and the initial conditions in an
  ! input file, and rewrite them in an output file.
  ! 
  subroutine initInteg(ficIn,ficOut, Hp,Ip, Z0)
     character(len=*),intent(in):: ficIn, ficOut
     type(hamparam),intent(in):: Hp
     type(intparam),intent(out):: Ip
     double precision,intent(out):: Z0(6*Hp%Nbod)


     double precision:: t0    ! initial time
     double precision:: tSpan ! duration
     double precision:: Dt    ! time step
     integer:: Nsave          ! number of timesteps between each save

     double precision:: X(6,Hp%Nbod), vbjs(3)

     integer:: Nreg, i, j

     namelist /integrationparam/ t0, tSpan, Dt, Nsave, X

     !-----------------------------------------------------------------------
     ! Reading the parameters
     open(newunit=Nreg,file=trim(ficIn),status="OLD",action="READ")
     read(Nreg,integrationparam)
     close(Nreg)

     ! Writing the parameters
     open(newunit=Nreg,file=trim(ficOut),status="REPLACE",action="WRITE")

     write(Nreg,'(A)')              "&integrationparam"
     write(Nreg,'(A,ES22.15,A)')    "     t0 = " , t0,","
     write(Nreg,'(A,ES22.15,A)')    "  tSpan = " , tSpan,","
     write(Nreg,'(A,ES22.15,A)')    "     Dt = " , Dt,","
     write(Nreg,'(A,I0,A)')         "  Nsave =  ", Nsave,","
     do j=1,Hp%Nbod
        write(Nreg,'(A,*(ES22.15,A))') "      X = " , (X(i,j),",",i=1,6)
     endDo
     write(Nreg,'(A)') "/"

     close(Nreg)

     ! NOTE: initial conditions taken from ephemerides, wrt central body
     ! canonical coordinates need barycentric velocities

     ! velocity of the barycenter
     vbjs=0.d0
     do j=1,Hp%Nbod
        vbjs=vbjs+Hp%mBod(j)*X(4:6,j)
     endDo
     vbjs=vbjs/Hp%Mtot

     ! initial conditions
     do j=1,Hp%Nbod
        ! heliocentric positions
        Z0(6*j-5:6*j-3)=X(1:3,j)
        ! barycentric momenta
        Z0(6*j-2:6*j)=(X(4:6,j)-vbjs)*Hp%mBod(j)
     endDo

     !-----------------------------------------------------------------------

     ! Parameters of the numerical integration
     Ip%t0      = t0
     Ip%tSpan   = tSpan
     Ip%Dt      = Dt
     Ip%Nsave   = Nsave

     !-----------------------------------------------------------------------

  end subroutine initInteg

!***************************************************************************************

  ! Deallocates the memory of p
  ! 
  subroutine freeHamParam(p)
     type(hamparam),intent(inOut):: p

     if(allocated(p%mBod))        deallocate(p%mBod)
     if(allocated(p%GmJmibi))     deallocate(p%GmJmibi)
     if(allocated(p%bi))          deallocate(p%bi)
     if(allocated(p%Gmimj))       deallocate(p%Gmimj)
     if(allocated(p%GmJmiRJ2J2))  deallocate(p%GmJmiRJ2J2)

  end subroutine freeHamParam

!***************************************************************************************
end module parameters
